from . import le

class OnePort:
    def __init__(self, e1, e2, i):
        self.e1 = e1
        self.e2 = e2
        self.i = i

class VSrc(OnePort):
    def __init__(self, v0, e1, e2, i):
        OnePort.__init__(self,e1,e2,i)
        self.v0 = v0
        self.equation = [(1,self.e1),(-1,self.e2), (-self.v0,None)]

class ISrc(OnePort):
    def __init__(self, i0, e1, e2, i):
        OnePort.__init__(self,e1,e2,i)
        self.i0 = i0
        self.equation = [(1,self.i),(-self.i0,None)]

class Resistor(OnePort):
    def __init__(self, r, e1, e2, i):
        OnePort.__init__(self,e1,e2,i)
        self.r = r
        self.equation = [(1,self.e1), (-1,self.e2), (-self.r,self.i)]

class VoltageSensor(OnePort):
    def __init__(self, e1, e2, i):
        OnePort.__init__(self,e1,e2,i)
        self.equation = [(1,self.i)]

class VCVS(OnePort):
    def __init__(self, sensor, e1, e2, i, K=1000000):
        OnePort.__init__(self,e1,e2,i)
        self.K = K
        self.equation = [(-self.K,sensor.e1), (self.K, sensor.e2), (1,self.e1),(-1,self.e2)]

def op_amp(ea1, ea2, Ia, eb1, eb2, Ib, K=1000000):
    vs = VoltageSensor(ea1,ea2,Ia)
    vcvs = VCVS(vs,eb1,eb2,Ib,K)
    return [vs,vcvs]

def flatten_list(l):
    out = []
    for i in l:
        if type(i) == list:
            out.extend(flatten_list(i))
        else:
            out.append(i)
    return out

def solve_circuit(componentList, GND):
    # flatten_list is necessary for lists that contain two-ports.
    # It has no effect on lists that contain just one-ports.
    components = flatten_list(componentList)

    equations = []

    currentsAtNode = {}
    # add component equations into set
    for component in components:
        equations.append(component.equation)

        # update the node->currents mapping
        for (sign,node) in [(1,component.e1),(-1,component.e2)]:
            currentsAtNode[node] = currentsAtNode.get(node,[])
            currentsAtNode[node].append((sign,component.i))
    
    # add KCL equations into set
    for node in currentsAtNode:
        if node == GND:
            continue
        equations.append(currentsAtNode[node])

    # equation for ground node
    equations.append([(1,GND)])

    return le.solve_equations(equations)
